      Subroutine cospl(x,y,n,a,b,c,d,ifail)
c
c  SCOPO: calcolo dei coefficienti della spline cubica naturale
c         interpolante n punti assegnati. 
c
c  PARAMETRI DI INPUT:
c
c  - x,  vettore di reali,  ascisse dei punti di interpolazione
c  - y,  vettore di reali,  ordinate dei punti di interpolazione
c  - n,            intero,  numero dei punti di interpolazione
c
c  PARAMETRI DI OUTPUT:
c
c  -     a,  vettore di reali,  coefficienti dei termini di grado
c                               3 degli n-1 polinomi interpolanti
c  -     b,  vettore di reali,  coefficienti dei termini di grado
c                               2 degli n-1 polinomi interpolanti
c  -     c,  vettore di reali,  coefficienti dei termini di grado
c                               1 degli n-1 polinomi interpolanti
c  -     d,  vettore di reali,  coefficienti dei termini di grado
c                               0 degli n-1 polinomi interpolanti
c  - ifail,            intero,  indicatore di errore
c
c  ROUTINE RICHIAMATE:
c
c  -  lut,  procedura,  fattorizzazione LU per matrici tridiagonali
c  - slut,  procedura,  risoluzione di un sistema tridiagonale
c
       external lut,slut
       
       integer n,ifail
       real x(n),y(n),a(n-1),b(n-1),c(n-1),d(n-1)
  
       logical ord
       
c  Controllo del numero di punti
        if (n .le. 2) then
           ifail=11
           goto 999
        endif
            
c  Ordinamento delle ascisse
       ord=.false.
       k=n
       do while (.not. ord)
c   10  continue
          ord=.true.
          do i=1,k-1    
             if (x(i) .gt. x(i+1)) then
                t=x(i)
                x(i)=x(i+1)
                x(i+1)=t
                ord=.false.
             endif  
          end do
          k=k-1   
      end do
c       if (.not. ord) goto 10

c  Controllo della non coincidenza delle ascisse
       do i=1,n-1
          if (x(i) .eq. x(i+1)) then
             ifail=12
             goto 999
          endif
       end do
   
c  Calcolo dei coefficienti e dei termini noti del sistema
c  avente come soluzione le derivate seconde della spline
c  nelle ascisse interne dei punti di interpolazione
       do i=1,n-1
          a(i)=x(i+1)-x(i)
       end do
       
       do i=2,n-2
          b(i)=a(i)
       end do
       
       do i=1,n-2
          c(i)=2*(a(i)+a(i+1))
       end do
       
       do i=1,n-2
          d(i+1)=6*((y(i)-y(i+1))/a(i) - (y(i+1)-y(i+2))/a(i+1))
       end do
       
c  Fattorizzazione LU della matrice dei coefficienti
       call lut(c,b(2),a(2),n-2,ifail)

c  Risoluzione del sistema tridiagonale
       call slut(c,b(2),a(2),d(2),n-2,ifail)

c  Derivata seconda nella prima ascissa
       d(1)=0       
       
c  Calcolo dei coefficienti in funzione delle derivate seconde
       do i=1,n-2
          c(i)=(y(i+1)-y(i))/a(i) - (d(i+1)+2*d(i))/6*a(i)
       end do
       c(n-1)=(y(n)-y(n-1))/a(n-1) - 2*d(n-1)/6*a(n-1)
       
       do i=1,n-1
          b(i)=d(i)/2
       end do
       
       do i=1,n-2
          a(i)=(d(i+1)-d(i))/6/a(i)
       end do
       a(n-1)=-d(n-1)/6/a(n-1)
       
       do i=1,n-1
          d(i)=y(i)
       end do
       
       ifail=0
      
  999  continue
      
      End
      stop