      real function espon(x,tol,ifail)
c
c  SCOPO: calcolo dell' esponenziale di un numero. 
c        
c  PARAMETRI DI INPUT:  
c 
c  -   x,  reale,  punto in cui calcolare la funzione
c  - tol,  reale,  tolleranza richiesta
c 
c  PARAMETRI DI OUTPUT:
c
c  - espon,   reale,  valore della funzione in x
c  - ifail,  intero,  indicatore d'errore
c   
c  ROUTINES RICHIAMATE:
c
c  - tolm,  funzione reale,  tolleranza minima relativa
c
       real tolm
c       external tolm
       
       real x,tol
       integer ifail

       integer n
 
c     Controllo della tolleranza        
       if (tol .lt. 0) then
          ifail=1
          goto 999
       endif
       
       vax=abs(x)
       sum=0
       add=1
       n=0
   10  continue
          sum=sum+add
          n=n+1
          add=add*(vax/n)
       if (add .ge. (tol*sum+tolm(sum))) goto 10
      
       if(x .ge. 0) then 
          espon=sum
       else
          espon=1/sum
       endif   
       
       ifail=0                                         
       
  999  continue    
       
       end     
      
      
      real function tolm(x)
c
c  SCOPO: calcolo della tolleranza minima relativa ad un numero x
c         di un sistema floating-point.
c        
c  PARAMETRI DI INPUT:  
c 
c  - x,  reale,  numero di cui calcolare la tolleranza relativa
c
c  PARAMETRI DI OUTPUT:
c
c  - tolm,  reale,  tolleranza minima reltiva ad x
c   
c  ROUTINES RICHIAMATE:
c
c  -  eps,  funzione reale,  epsilon macchina
c  - minr,  funzione reale,  minimo rappresentabile
c
       real eps,minr
       external eps,minr
       
       real e,m,x
           
       e=eps()
       m=minr()
       if (abs(x).gt.m/(2*e)) then
          tolm=abs(x)*2*e
       else 
          tolm=m
       endif

      end 
      
      real function minr()
c
c  SCOPO: calcolo del minimo numero positivo di un sistema 
c         floating-point.
c        
c  PARAMETRI DI INPUT:  
c 
c  - nessuno
c
c  PARAMETRI DI OUTPUT:
c
c  - minr,  reale,  minimo numero positivo rappresentabile
c   
c  ROUTINES RICHIAMATE:
c
c  - eps,  funzione reale,  epsilon macchina 
c            
       real eps
c       external eps
       
       real xold,xnew
 
       xnew=1.0
   10  if (xnew.eq.0) goto 20
          xold=xnew
          xnew=xold/2
       goto 10
   20  continue
       
c     Operazione di shift left della mantissa al fine di
c     ottenere il minimo rappresentabile del sistema
c     floating-point normalizzato
       minr=xold/(2*eps())

      End  
      
      
      real function eps ()
c
c  SCOPO: calcolo dell'epsilon macchina di un sistema floating-point.
c        
c  PARAMETRI DI INPUT:  
c 
c  - nessuno
c
c  PARAMETRI DI OUTPUT:
c
c  - eps,  reale,  epsilon macchina
c   
c  ROUTINES RICHIAMATE:
c
c  - nessuna 
c
       real e,e1,e2        
       
       e1=1
       e2=e1+1
       
   10  continue  
       if (e2 .eq. 1) goto 20
          e=e1
          e1=e1/2
          e2=e1+1
       goto 10
   20  continue
       
       eps=e
      
      End 
      
      Program espond
c     Programma dimostrativo della function espon
      
      real espon
      external espon
      
      integer dim
      parameter (dim=100)
       
      real x(dim),esp(dim),t
      integer n,ifail
      character*12 fin,fout
      
      write(*,20)'Numero di punti (max ',dim,') ='
      read(*,*)n
      print*
      print*,'Nome del file di input ='
      read(*,'(A)')fin
      open(11,FILE=fin,STATUS='OLD')
      read(11,*)(x(i),i=1,n) 
      close(11)
      print*
      print*,'Tolleranza richiesta ='
      read(*,*)t                               
      print*
      
      esp(1)=espon(x(1),t,ifail)
      if (ifail.eq.1) stop 'Errore! tolleranza non valida'
      do 10 i= 2,n
         esp(i)=espon(x(i),t,ifail)
   10 continue        

      print*,'Nome del file di output ='
      read(*,'(A)')fout
      open(12,FILE=fout)
      write(12,30)(esp(i),i=1,n)
      close(12)                    
      print*
      print*,'Valori calcolati ='
      write(*,30)(esp(i),i=1,n)
        
   20 format (1X,A,I3,A)
   30 format (1X,E14.7)      
   
      End    