      Subroutine fzero(f,a,b,tolx,tolf,nmax,z,fz,nval,ifail)
c
c  SCOPO: risoluzione dell'equazione f(x)=0 (ovvero calcolo
c         dello zero di f) in un intervallo compatto negli
c         estremi del quale la f, funzione reale di una
c         variabile reale, assume valori di segno opposto.
c 
c  PARAMETRI DI INPUT:  
c 
c  -      f,  funzione reale,  funzione di cui calcolare lo zero
c  -      a,           reale,  estremo sinistro dell'intervallo
c                              contenente lo zero
c  -      b,           reale,  estremo destro dell'intervallo
c                              contenente lo zero
c  -   tolx,           reale,  tolleranza relativa per lo zero
c  -   tolf,           reale,  tolleranza assoluta per la funzione
c  -   nmax,          intero,  numero massimo di valutazioni
c
c  PARAMETRI DI OUTPUT:
c
c  -     z,   reale,  zero della funzione (=soluzione calcolata)    
c  -    fz,   reale,  valore della funzione (=residuo)
c  -  nval,  intero,  numero di valutazioni occorse
c  - ifail,  intero,  indicatore d'errore
c   
c  ROUTINES RICHIAMATE:
c
c  - tolm,  funzione reale,  tolleranza minima relativa
c
       real eps,tolm
       external eps,tolm
                        
       real f
       external f
                        
       real a,b,tolx,tolf,z,fz
       integer nmax,nval,ifail
        
c  Controllo della tolleranza relativa
       if (tolx .lt. 0  .or.  tolx .ge. 1) then
          ifail=1
          goto 999
       endif

c  Controllo della tolleranza assoluta
       if (tolf .lt. 0) then
          ifail=2
          goto 999
       endif

c  Controllo del numero di valutazioni di funzione
       if (nmax .lt. 2) then
          ifail=16
          goto 999
       endif
      
c  Controllo del segno della funzione negli estremi
       if (f(a)*f(b) .ge. 0) then
          ifail=22
          goto 999
       endif
       
c  Inizializzazioni                
       e=eps()
       x0=a
       x1=b
       y1=x0
       y0=y1*(1+2*e)
       ym1=y0
       f0=f(x0)
       f1=f(x1)     
       nval=2
       ifail=0

c  Ciclo a condizione: termina se si  raggiunta una delle
c  due tolleranze o il numero massimo di valutazioni       
   10  continue
       xy=x1-y1
       if (abs(xy) .le. tolx*abs(x1)+tolm(x1)  
     %    .or.  abs(f1) .le. tolf+e)           goto 20

c     Controllo del numero di valutazioni          
          if (nval .ge. nmax) then
             ifail=19
             goto 20
          endif

c     Controlla che un estremo non si ripeta troppe volte
          if (y1 .ne. ym1) then
             d=(f1/f0)*(x0-x1)/(1-f1/f0)
c        Controlla che la secante non esca dall'intervallo [x1,y1]                          
             if (d*xy .lt. 0  .or.  abs(d) .gt. abs(xy)) d=0.5*xy
          else
             d=0.5*xy
          endif

          x0=x1
          f0=f1
          x1=x1-d
          f1=f(x1)
          nval=nval+1
          ym1=y0
          y0=y1
          if (f0*f1 .lt. 0) y1=x0

       goto 10
   20  continue    
                                               
c  Soluzione calcolata   
       z=x1
c  Residuo
       fz=f1
         
  999  continue       
         
      End 
      stop         