      Subroutine lups(a,lda,n,ipiv,s,ifail,log)
c
c  SCOPO: fattorizzazione di una matrice quadrata A nella forma
c         P*A=L*U, dove P  la matrice delle permutazioni derivante
c         dall' applicazione del pivoting scalato, L una matrice
c         triangolare inferiore a diagonale unitaria e U triangolare
c         superiore.
c 
c  PARAMETRI DI INPUT:  
c 
c  -   a,  matrice di reali,  matrice da fattorizzare
c  - lda,            intero,  leading dimension di a dichiarata 
c                             nel programma chiamante
c  -   n,            intero,  ordine della matrice
c 
c  PARAMETRI DI OUTPUT:
c
c  -     a,   matrice di reali,  permutate le righe secondo ipiv
c                                restituisce i fattori L ed U 
c  -  ipiv,  vettore di interi,  indici delle righe di a da per-
c                                mutare per ottenere L ed U
c  -     s,   vettore di reali,  norma infinito delle righe di A
c  - ifail,             intero,  indicatore di errore
c   
c  ROUTINES RICHIAMATE:
c
c  - eps,  funzione reale,  epsilon macchina
c  
       real eps
       external eps
       
       integer lda,n,ipiv(n),ifail
       real a(lda,n),s(n)
              
c  Controllo della dimensione
       
       if (n .le. 0 .or. n .gt. lda) then
          ifail=3                        
          goto 999
       endif                             
              
       e=eps()                                           
       
c  Inizializzazione di ipiv e ricerca dei massimi di riga 
       write(log,*)
       write(log,*)'inizializ. di ipiv e ricerca dei massimi di riga'
       write(log,*)'ricerca del massimo tra s(i) e |a(i,j)|'
       
       do 20 i=1,n
          ipiv(i)=i      
          s(i)=a(i,1)
c
          write(log,*)
          write(log,*)'ipiv(i)= ',ipiv(i)
          write(log,*)'s(i)= ',s(i) 
          
          do 10 j=2,n 
c             write(log,*)s(i),abs(a(i,j))
             s(i)=amax1(s(i),abs(a(i,j)))
   10     continue
          write(log,*)'max=',s(i)

c     Controllo della singolarit      
          if (s(i) .lt. e) then
             ifail=4                            
             goto 999
          endif
   20  continue                             
       
       write(log,*)
       write(log,*)'vettore pivot'
       write(log,*)(ipiv(i),i=1,n)
       write(log,*)'massimi di riga'
       write(log,*)(s(i),i=1,n)
       write(log,*)

       
       do 70 k=1,n
       write(log,*)'ricerca del pivot scalato (piv/s) '  
         
c     Ricerca del pivot     

          ps=abs(a(ipiv(k),k))/s(k)
          m=k
          write(log,*)'pivot (ps)=',ps,' in riga=',m
          write(log,*)'cerca un pivot maggiore di ps sulla colonna ',k
          
          do 30 i=k+1,n                                       
             write(log,*)'as=a(ipiv(',i,'),',k,')/s(',i,')'
             write(log,*)a(ipiv(i),k),s(i) 
             
             as=abs(a(ipiv(i),k))/s(i)   
             
             write(log,*)'riga:',ipiv(i),' colonna:',k,' as=',as
             if (as .gt. ps) then
                write(log,*)'trovato pivot maggiore'
                write(log,*)'vecchio pivot=',ps,' in riga:',m,
     %                                          ' colonna:',k
                ps=as
                m=i                         
                write(log,*)'nuovo pivot=',ps,' in riga:',m,
     %                                          ' colonna:',k             
             endif
             
   30     continue
c
          write(log,*)'il pivot ',ps,'  in ',m,' riga ',k, ' colonna'
          write(log,*)
          
c          
c     Controllo della singolarit
          if (ps .lt. 0) then
             ifail=4                          
             write(log,*)'pivot singolare'
             goto 999
          endif                          
      
c     Scambio virtuale 
          if (m .ne. k) then
             write(log,*)'scambio virtuale'
             write(log,*)'vecchio vett. pivot',(ipiv(i),i=1,n)
             write(log,*)'scambio gli elementi di posizione',m,k
             temp=ipiv(m)
             ipiv(m)=ipiv(k)
             ipiv(k)=temp  
             write(log,*)
             write(log,*)'nuovo vett. pivot',(ipiv(i),i=1,n)
             write(log,*)
       
          endif

         write(log,*)'calcolo dei moltiplicatori'  
          
c     Calcolo dei moltiplicatori
          do 40 i=k+1,n
                a(ipiv(i),k)=a(ipiv(i),k)/a(ipiv(k),k)
                write(log,*)'a(ipiv(',i,'),',k,')=',a(ipiv(i),k)
   40     continue       
      
          write(log,*)'matrice attuale'
          do i=1,n
              write(log,*)(a(i,j),j=1,n)
          end do
       
          do 60 j=k+1,n
             do 50 i=k+1,n  
             write(log,*)'a(ipiv(',i,'),',j,')=a(ipiv(',i,'),',j,')-
     %                     a(ipiv(',i,'),',k,')*a(ipiv(',i,'),',j,')='           
                a(ipiv(i),j)=a(ipiv(i),j)-a(ipiv(i),k)*a(ipiv(k),j)
            
                write(log,*)i,j,a(ipiv(i),j)
   50        continue                                           
             write(log,*)
   60     continue 
   
       write(log,*)'matrice attuale'
       do i=1,n
          write(log,*)(a(ipiv(i),j),j=1,n)
       end do
       write(log,*)
  
  
   70  continue
         
       ifail=0
       
  999  continue

      stop
	end      
      
      
      


      
   
   
   
