      Subroutine ode45 (f,a,b,y0,m,tol,n,x,y,ifail)
c
c  SCOPO: risoluzione di un'equazione differenziale ordinaria
c         in un intervallo compatto.
c 
c  PARAMETRI DI INPUT:  
c  
c  -    f,  funzione reale,  termine noto dell'equazione
c  -    a,           reale,  estremo sinistro dell'intervallo 
c  -    b,           reale,  estremo destro dell'intervallo
c  -   y0,           reale,  valore iniziale in a
c  -    m,          intero,  numero iniziale di intervalli
c  -  tol,           reale,  tolleranza richiesta
c  -    n,          intero,  numero massimo di valori da calcolare
c 
c  PARAMETRI DI OUTPUT:
c
c  -      x,  vettore di reali,  punti di griglia
c  -      y,  vettore di reali,  valori della soluzione in x
c  -  ifail,            intero,  indicatore d'errore
c
c  ROUTINES RICHIAMATE:
c
c  -  tolm,  funzione reale,  tolleranza minima relativa
c
       real tolm
       external tolm
       
       integer m,n,ifail
       real a,b,y0,tol,x(n),y(n)
       
       real f
       external f
                  
       real cost
       parameter (cost=64)
       
       real hmin,hmax,k(6),c(21),r
       logical flag
                         
       ifail=0                  

c  Controllo degli estremi
       if (b-a .lt. 0) then
          ifail=23
          goto 999
       endif

c  Controllo del numero iniziale di intervalli
       if (m .le. 0) then
          ifail=24
          goto 999
       endif
       
c  Controllo della tolleranza assoluta
       if (tol .lt. 0) then
          ifail=2
          goto 999
       endif

c  Controllo del numero massimo di valori da calcolare
       if (n .le. 0) then
          ifail=11
          goto 999
       endif
       
c  Inizializzazione delle variabili e delle costanti
       i=1
       x(i)=a
       y(i)=y0
       h=(b-a)/m
       hmin=h/cost
       hmax=cost*h
       c(1)=3./8
       c(2)=3./32
       c(3)=9./32
       c(4)=12./13
       c(5)=1932./2197
       c(6)=7002./2197
       c(7)=7296./2197
       c(8)=439./216
       c(9)=3680./513       
       c(10)=845./4104
       c(11)=8./27
       c(12)=3544./2565
       c(13)=1859./4104
       c(14)=11./40
       c(15)=16./135
       c(16)=6656./12825
       c(17)=28561./56430
       c(18)=9./50
       c(19)=2./55
       c(20)=128./4275
       c(21)=2197./75240
       flag=.true.

       if (n .eq. 1  .or.  b-a .eq. 0) goto 999
c  N.B.: la soluzione coincide con la condizione iniziale

c  Ciclo a condizione: termina se il passo h  minore del passo
c  minimo hmin oppure l'ultima ascissa  esterna all'intervallo
   10  continue      

c    Controllo del raggiungimento del numero massimo di valori          
         if (i .eq. n) then
            ifail=25
            goto 999
         endif                                                

c    Calcolo delle costanti K1,..,K6 e della nuova approssimazione
         k(1)=h*f(x(i),y(i))
         k(2)=h*f(x(i)+h/4,y(i)+k(1)/4)
         k(3)=h*f(x(i)+c(1)*h,y(i)+c(2)*k(1)+c(3)*k(2))
         k(4)=h*f(x(i)+c(4)*h,y(i)+c(5)*k(1)-c(6)*k(2)
     %            +c(7)*k(3)) 
         k(5)=h*f(x(i)+h,y(i)+c(8)*k(1)-8*k(2)+c(9)*k(3)
     %            -c(10)*k(4))
         k(6)=h*f(x(i)+h/2,y(i)-c(11)*k(1)+2*k(2)-c(12)*k(3)
     %            +c(13)*k(4)-c(14)*k(5))
                                   
         y(i+1)=y(i)+c(15)*k(1)+c(16)*k(3)+c(17)*k(4)
     %         -c(18)*k(5)+c(19)*k(6)
  
c    Calcolo della differenza tra la soluzione del metodo di ordine 4 
c    e quella del metodo di ordine 5
         r=abs(k(1)/360-c(20)*k(3)-c(21)*k(4)+k(5)/50
     %         +c(19)*k(6))
    
c    Controllo del raggiungimento della tolleranza richiesta         
         if (r .le. tol*h+tolm(y(i+1))) then
            x(i+1)=x(i)+h
            i=i+1
         endif

c    Calcolo del fattore di proporzionalit
         q=0.84*(tol*h/r)**0.25

c    Modifica del passo h in funzione del valore di q
         if (q .le. 0.1) then
            h=0.1*h
         elseif (q .ge. 4) then
            h=4*h
         else 
            h=q*h
         endif
         
c    Controllo del nuovo passo ed eventuale modifica                    
         if (h .gt. hmax) h=hmax
         
c    Controllo del punto di griglia 
         if (x(i) .ge. b) then 
            flag=.false.
         elseif (x(i)+h .ge. b) then
            h=b-x(i)
         elseif (h .lt. hmin) then
            ifail=26
            flag=.false.
         endif   

       if (flag) goto 10
       
       n=i
       
  999  continue    
      
	End
        stop