      Subroutine quad(f,a,b,tol,hmin,nmax,q,e,nval,ifail)
c
c  SCOPO: calcolo dell'integrale definito di una funzione
c         reale di una variabile reale.
c 
c  PARAMETRI DI INPUT:  
c 
c  -      f,  funzione reale,  funzione integranda
c  -      a,           reale,  estremo sinistro 
c                              dell'intervallo d'integrazione
c  -      b,           reale,  estremo destro 
c                              dell'intervallo d'integrazione
c  -    tol,           reale,  tolleranza di calcolo richiesta
c  -   hmin,           reale,  passo minimo d'integrazione
c  -   nmax,          intero,  numero massimo di valutazioni
c
c  PARAMETRI DI OUTPUT:
c
c  -     q,   reale,  stima dell'integrale definito di f    
c  -     e,   reale,  stima dell'errore di calcolo
c  -  nval,  intero,  numero minimo di valutazioni occorse
c  - ifail,  intero,  indicatore d'errore
c   
c  ROUTINES RICHIAMATE:
c
c  -  eps,  funzione reale,  epsilon macchina
c  - simp,  funzione reale,  calcolo dell'integrale mediante
c                            la formula di Cavalieri-Simpson
c  - tolm,  funzione reale,  tolleranza minima relativa
c
       real simp,eps,tolm
       external simp,eps,tolm
                        
       external f
                        
       integer dim
       parameter (dim=25)                  
       
       real a,b,tol,hmin,q,e
       integer nmax,nval,ifail
        
       real h,stack(dim,3),c,d,pm,s1,s2,eloc
       integer n
        
c  Controllo della tolleranza assoluta
       if (tol .lt. 0) then
          ifail=2
          goto 999
       endif

c  Calcolo dell'ampiezza dell'intervallo di integrazione
       h=abs(b-a)
       
c  Inizializzazione delle variabili di uscita
       q=0
       e=0
       nval=0
       ifail=0

c  Controllo del numero di valutazioni massime eseguibili
       if (nmax .lt. 5) then
          ifail=16
          goto 999
       endif
      
c  Controllo degli estremi dell'intervallo
       if (b-a .eq. 0) goto 999

c  Controllo del passo minimo d'integrazione
       if (hmin .le. 0 .or. hmin .ge. h) then
          ifail=17
          goto 999
       endif
       if (hmin .lt. 8*eps()*h) hmin=8*eps()*h
       
c  Inizializzazione dello stack
c  N.B.: se a>b l'integrale viene calcolato tra b e a
       stack(1,1)=min(a,b)
       stack(1,2)=max(a,b)
       stack(1,3)=simp(f,stack(1,1),stack(1,2))
       
       n=1
       nval=nval+3
       
c  Ciclo a condizione: termina se lo stack  vuoto o si 
c  raggiunge il numero massimo di valutazioni 
   10  continue

c  Prelievo dallo stack degli estremi dell'intervallo corrente
c  d'integrazione e dell'integrale relativo ad esso
       c=stack(n,1)
       d=stack(n,2)
       s1=stack(n,3)
       n=n-1
       
c  Calcolo del punto medio 
       if (c*d .lt. 0) then
          pm=(c+d)/2      
       else
          pm=c+(d-c)/2
       endif

c  Calcolo della somma dei due integrali relativi alle
c  due met dell'intervallo corrente d'integrazione
       s2=simp(f,c,pm)+simp(f,pm,d)

c  Calcolo dell'errore locale       
       eloc=abs(s1-s2)/15
       
c  Controllo dell'errore locale e del passo d'integrazione
       if (eloc .lt. tol*(d-c)/h+tolm(s1) .or. d-c .le. hmin) then
          q=q+s2
          e=e+eloc
       else                           

c  Controllo della lunghezza dello stack  
          if (n .gt. dim-2) then
             ifail=18
             n=0
          else   
             n=n+1
             stack(n,1)=pm
             stack(n,2)=d
             stack(n,3)=simp(f,pm,d)
             n=n+1
             stack(n,1)=c
             stack(n,2)=pm       
             stack(n,3)=simp(f,c,pm)
          endif
       endif
       
c  Calcolo del numero di valutazioni 
       nval=nval+2
          
       if (n .gt. 0 .and. nval .lt. nmax) goto 10

c  Valore dell'integrale
c  N.B.: se a<b si moltiplica per -1
c  l'integrale definito tra b ed a
       q=sign(1.0,b-a)*q

c  Controllo dello stack                               
       if (n .gt. 0) ifail=19
       
  999  continue
  
      End
      