      Subroutine sspar(r,c,irc,nrc,n,b,tolr,resr,nmax,x0,x,ifail)
c
c  SCOPO: risoluzione di un sistema lineare sparso.
c
c  PARAMETRI DI INPUT:                                      
c                              
c  -    r,   vettore di reali,  elementi non nulli della matrice
c                               dei coefficienti del sistema
c  -    c,  vettore di interi,  indici di colonna nella matrice
c                               dei coefficienti degli elementi di r
c  -  irc,  vettore di interi,  posizioni in r del primo elemento
c                               non nullo di ciascuna riga
c  -  nrc,             intero,  numero di elementi non nulli
c  -    n,             intero,  ordine del sistema
c  -    b,   vettore di reali,  vettore dei termini noti
c  - tolr,              reale,  tolleranza relativa richiesta
c  - resr,              reale,  maggiorazione per il residuo
c                               relativo del sistema
c  - nmax,             intero,  numero massimo di valutazioni
c  -   x0,   vettore di reali,  approssimazione iniziale della 
c                               soluzione
c
c  PARAMETRI DI OUTPUT:
c
c  -     x,  vettore di reali,  soluzione del sistema
c  - ifail,            intero,  indicatore d'errore
c
c  ROUTINES RICHIAMATE:
c
c  - tolm,  funzione reale,  tolleranza minima relativa
c
       integer nrc,n,c(nrc),irc(n+1),nmax,ifail
       real r(nrc),tolr,resr,b(n),x0(n),x(n)
       
       real tolm
       external tolm
      
       real maxn,maxd
       logical flag
              
c  Controllo delle dimensioni dei vettori
       if (nrc .le. 0  .or.  n .le. 0) then
          ifail=3
          goto 999
       endif
      
c  Controllo delle tolleranze relative
       if (tolr .lt. 0  .or.  tolr .ge. 1  .or.  
     %     resr .lt. 0  .or.  resr .ge. 1      ) then
          ifail=1
          goto 999
       endif
      
c  Controllo degli elementi diagonali
       do 30 i=1,n
          l=irc(i)                                       
   10     continue          
          if (c(l) .ge. i  .or.  l .eq. irc(i+1)-1) goto 20
             l=l+1
          goto 10
   20     continue
          if (c(l) .eq. i .and. r(l) .eq. 0 .or. c(l) .gt. i) then
             ifail=10
             goto 999
          endif
   30  continue
   
c  Inizializzazione
       flag=.false.
       k=0  

   40  continue

c  Controllo del raggiungimento del numero massimo di iterazioni
         if (k .eq. nmax) then
            ifail=9
            goto 999
         endif                                                  
         
         l=0
         do 60 i=1,n
            x(i)=b(i)
            num=irc(i+1)-irc(i)
            do 50 j=1,num
               l=l+1
               if (c(l) .lt. i) then
                  x(i)=x(i)-r(l)*x(c(l))
               elseif (c(l) .eq. i) then
                  d=r(l)
               elseif (c(l) .gt. i) then
                  x(i)=x(i)-r(l)*x0(c(l))
               endif
   50       continue            
            x(i)=x(i)/d
   60    continue            

         k=k+1
   
c    Calcolo della norma infinito ||x-x0|| e ||x||
         maxn=abs(x(1)-x0(1))
         maxd=abs(x(1))
         do 70 i=2,n
            temp=abs(x(i)-x0(i))
            if (temp .gt. maxn) maxn=temp
            temp=abs(x(i))
            if (temp .gt. maxd) maxd=temp
   70    continue
         
c    Test sulla convergenza
         if (maxn .le. tolr*maxd+tolm(maxd)) then
            flag=.true.
         else
       
c       Calcolo della norma infinito ||b-A*x|| e ||b||
            maxn=abs(b(1)-r(1)*(x(1)-x0(1)))
            maxd=abs(b(1))
            do 100 i=2,n  

c          Ricerca dell'indice dell'elemento diagonale   
               l=irc(i)
   80          continue
               if (c(l) .eq. i) goto 90
                  l=l+1
               goto 80
   90          continue         
    
               temp=abs(b(i)-r(l)*(x(i)-x0(i)))
               if (temp .gt. maxn) maxn=temp
               temp=abs(b(i))
               if (temp .gt. maxd) maxd=temp
  100       continue

c       Test sulla bont del risultato
            if (maxn .le. resr*maxd+tolm(maxd)) flag=.true.
         endif
          
c    Copia di x in x0
         do 110 i=1,n
            x0(i)=x(i)
  110    continue
  
       if (.not. flag) goto 40
       
       ifail=0
       
  999  continue
  
      End
	stop